// Fichier Game.java
// Contient la logique principale du jeu 2048 : gestion du plateau, mouvements, fusions, score.
// Cette classe est maintenant indépendante du contexte Android et de la persistance des données.
/*
    Fonctions principales :
    - board : Matrice 2D (int[][]) représentant la grille du jeu.
    - currentScore : Suivi du score de la partie en cours.
    - highestScore : Stocke le meilleur score global (reçu de l'extérieur via setHighestScore).
    - addNewTile() : Ajoute une nouvelle tuile aléatoire sur une case vide.
    - pushUp(), pushDown(), pushLeft(), pushRight() : Gèrent la logique de déplacement et de fusion, retournent un booléen indiquant si le plateau a changé.
    - getHighestTileValue() : Retourne la valeur de la plus haute tuile sur le plateau.
    - États gameWon, gameOver : Indiquent si la partie est gagnée ou terminée.
    - Méthodes pour vérifier les conditions de victoire et de fin de partie.
    - toString(), deserialize() : Sérialisent/désérialisent l'état essentiel du jeu (plateau, score courant) pour la sauvegarde externe.

    Relations :
    - MainActivity : Crée une instance de Game, appelle ses méthodes (pushX, addNewTile, getters), lui fournit le meilleur score global via setHighestScore, et utilise son état pour l'affichage et la sauvegarde.
*/
package legion.muyue.best2048;

import androidx.annotation.NonNull;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

public class Game {

    private int[][] board;
    private final Random randomNumberGenerator;
    private int currentScore = 0;
    private int highestScore = 0; // Stocke le HS fourni par MainActivity
    private static final int BOARD_SIZE = 4;
    private boolean gameWon = false;
    private boolean gameOver = false;

    /**
     * Constructeur pour une nouvelle partie.
     * Initialise un plateau vide, définit le score à 0 et ajoute deux tuiles initiales.
     */
    public Game() {
        this.randomNumberGenerator = new Random();
        // Le highScore sera défini par MainActivity après l'instanciation.
        initializeNewBoard();
    }

    /**
     * Constructeur utilisé lors de la restauration d'une partie sauvegardée.
     * @param board Le plateau de jeu restauré.
     * @param score Le score courant restauré.
     */
    public Game(int[][] board, int score) {
        this.board = board;
        this.currentScore = score;
        this.randomNumberGenerator = new Random();
        // Le highScore sera défini par MainActivity après l'instanciation.
        checkWinCondition(); // Recalcule l'état basé sur le plateau chargé
        checkGameOverCondition();
    }

    // --- Getters / Setters ---

    /**
     * Retourne la valeur de la cellule aux coordonnées spécifiées.
     * @param row Ligne de la cellule (0-based).
     * @param column Colonne de la cellule (0-based).
     * @return Valeur de la cellule, ou 0 si indices invalides (sécurité).
     */
    public int getCellValue(int row, int column) {
        if (row < 0 || row >= BOARD_SIZE || column < 0 || column >= BOARD_SIZE) { return 0; }
        return this.board[row][column];
    }

    /**
     * Définit la valeur de la cellule aux coordonnées spécifiées.
     * @param row Ligne de la cellule (0-based).
     * @param col Colonne de la cellule (0-based).
     * @param value Nouvelle valeur.
     */
    public void setCellValue(int row, int col, int value) {
        if (row < 0 || row >= BOARD_SIZE || col < 0 || col >= BOARD_SIZE) { return; }
        this.board[row][col] = value;
    }

    /** @return Le score actuel de la partie. */
    public int getCurrentScore() { return currentScore; }

    // public void setCurrentScore(int currentScore) { this.currentScore = currentScore; } // Setter interne si nécessaire

    /** @return Le meilleur score connu par cet objet Game (synchronisé par MainActivity). */
    public int getHighestScore() { return highestScore; }

    /**
     * Définit le meilleur score global connu. Appelé par MainActivity après chargement
     * des préférences ou après une mise à jour du score.
     * @param highScore Le meilleur score connu.
     */
    public void setHighestScore(int highScore) { this.highestScore = highScore; }

    /** @return L'état de victoire de la partie (true si >= 2048 atteint). */
    public boolean isGameWon() { return gameWon; }

    public void setGameWon(boolean gameWon) { this.gameWon = gameWon; }

    /** @return L'état de fin de partie (true si aucun mouvement possible). */
    public boolean isGameOver() { return gameOver; }

    public void setGameOver(boolean gameOver) { this.gameOver = gameOver; }

    /** @return Une copie du plateau de jeu actuel (pour la sauvegarde externe). */
    public int[][] getBoard() {
        // Retourne une copie pour éviter modifications externes accidentelles
        int[][] copy = new int[BOARD_SIZE][BOARD_SIZE];
        for(int i=0; i<BOARD_SIZE; i++) {
            System.arraycopy(this.board[i], 0, copy[i], 0, BOARD_SIZE);
        }
        return copy;
    }

    /**
     * Initialise ou réinitialise le plateau pour une nouvelle partie.
     * Met le score à 0 et ajoute deux tuiles aléatoires.
     */
    private void initializeNewBoard() {
        this.board = new int[BOARD_SIZE][BOARD_SIZE];
        this.currentScore = 0;
        this.gameWon = false;
        this.gameOver = false;
        addNewTile();
        addNewTile();
    }

    // --- Logique du Jeu ---

    /**
     * Ajoute une nouvelle tuile (selon les probabilités définies)
     * sur une case vide aléatoire du plateau. Ne fait rien si le plateau est plein.
     */
    public void addNewTile() {
        if (!hasEmptyCell()) { return; }
        List<int[]> emptyCells = new ArrayList<>();
        for (int row = 0; row < BOARD_SIZE; row++) {
            for (int col = 0; col < BOARD_SIZE; col++) {
                if (board[row][col] == 0) { emptyCells.add(new int[]{row, col}); }
            }
        }
        if (!emptyCells.isEmpty()) {
            int[] randomCell = emptyCells.get(randomNumberGenerator.nextInt(emptyCells.size()));
            int value = generateRandomTileValue();
            setCellValue(randomCell[0], randomCell[1], value);
        }
    }

    /**
     * Génère aléatoirement la valeur d'une nouvelle tuile (2, 4, 8, etc.)
     * selon des probabilités prédéfinies.
     * @return La valeur de la nouvelle tuile.
     */
    private int generateRandomTileValue() {
        int randomValue = randomNumberGenerator.nextInt(10000);
        if (randomValue < 8540) return 2;  // ~85%
        if (randomValue < 9740) return 4;  // ~12%
        if (randomValue < 9940) return 8;  // ~2%
        if (randomValue < 9990) return 16; // ~0.5%
        // ... (autres probabilités)
        if (randomValue < 9995) return 32;
        if (randomValue < 9998) return 64;
        if (randomValue < 9999) return 128;
        return 256;
    }

    /**
     * Tente de déplacer et fusionner les tuiles vers le HAUT.
     * Met à jour le score interne en cas de fusion.
     * Vérifie les conditions de victoire/défaite après le mouvement.
     * @return true si au moins une tuile a bougé ou fusionné, false sinon.
     */
    public boolean pushUp() {
        boolean boardChanged = false; boolean[] hasMerged = new boolean[BOARD_SIZE];
        for (int col = 0; col < BOARD_SIZE; col++) {
            hasMerged = new boolean[BOARD_SIZE];
            for (int row = 1; row < BOARD_SIZE; row++) {
                if (getCellValue(row, col) != 0) {
                    int currentValue = getCellValue(row, col); int currentRow = row;
                    while (currentRow > 0 && getCellValue(currentRow - 1, col) == 0) { setCellValue(currentRow - 1, col, currentValue); setCellValue(currentRow, col, 0); currentRow--; boardChanged = true; }
                    if (currentRow > 0 && getCellValue(currentRow - 1, col) == currentValue && !hasMerged[currentRow - 1]) {
                        int newValue = getCellValue(currentRow - 1, col) * 2; setCellValue(currentRow - 1, col, newValue); setCellValue(currentRow, col, 0);
                        currentScore += newValue; hasMerged[currentRow - 1] = true; boardChanged = true;
                    }
                }
            }
        } checkWinCondition(); checkGameOverCondition(); return boardChanged;
    }

    /**
     * Tente de déplacer et fusionner les tuiles vers le BAS.
     * @return true si changement, false sinon.
     */
    public boolean pushDown() {
        boolean boardChanged = false; boolean[] hasMerged = new boolean[BOARD_SIZE];
        for (int col = 0; col < BOARD_SIZE; col++) {
            hasMerged = new boolean[BOARD_SIZE];
            for (int row = BOARD_SIZE - 2; row >= 0; row--) {
                if (getCellValue(row, col) != 0) {
                    int currentValue = getCellValue(row, col); int currentRow = row;
                    while (currentRow < BOARD_SIZE - 1 && getCellValue(currentRow + 1, col) == 0) { setCellValue(currentRow + 1, col, currentValue); setCellValue(currentRow, col, 0); currentRow++; boardChanged = true; }
                    if (currentRow < BOARD_SIZE - 1 && getCellValue(currentRow + 1, col) == currentValue && !hasMerged[currentRow + 1]) {
                        int newValue = getCellValue(currentRow + 1, col) * 2; setCellValue(currentRow + 1, col, newValue); setCellValue(currentRow, col, 0);
                        currentScore += newValue; hasMerged[currentRow + 1] = true; boardChanged = true;
                    }
                }
            }
        } checkWinCondition(); checkGameOverCondition(); return boardChanged;
    }

    /**
     * Tente de déplacer et fusionner les tuiles vers la GAUCHE.
     * @return true si changement, false sinon.
     */
    public boolean pushLeft() {
        boolean boardChanged = false; boolean[] hasMerged = new boolean[BOARD_SIZE];
        for (int row = 0; row < BOARD_SIZE; row++) {
            hasMerged = new boolean[BOARD_SIZE];
            for (int col = 1; col < BOARD_SIZE; col++) {
                if (getCellValue(row, col) != 0) {
                    int currentValue = getCellValue(row, col); int currentCol = col;
                    while (currentCol > 0 && getCellValue(row, currentCol - 1) == 0) { setCellValue(row, currentCol - 1, currentValue); setCellValue(row, currentCol, 0); currentCol--; boardChanged = true; }
                    if (currentCol > 0 && getCellValue(row, currentCol - 1) == currentValue && !hasMerged[currentCol - 1]) {
                        int newValue = getCellValue(row, currentCol - 1) * 2; setCellValue(row, currentCol - 1, newValue); setCellValue(row, currentCol, 0);
                        currentScore += newValue; hasMerged[currentCol - 1] = true; boardChanged = true;
                    }
                }
            }
        } checkWinCondition(); checkGameOverCondition(); return boardChanged;
    }

    /**
     * Tente de déplacer et fusionner les tuiles vers la DROITE.
     * @return true si changement, false sinon.
     */
    public boolean pushRight() {
        boolean boardChanged = false; boolean[] hasMerged = new boolean[BOARD_SIZE];
        for (int row = 0; row < BOARD_SIZE; row++) {
            hasMerged = new boolean[BOARD_SIZE];
            for (int col = BOARD_SIZE - 2; col >= 0; col--) {
                if (getCellValue(row, col) != 0) {
                    int currentValue = getCellValue(row, col); int currentCol = col;
                    while (currentCol < BOARD_SIZE - 1 && getCellValue(row, currentCol + 1) == 0) { setCellValue(row, currentCol + 1, currentValue); setCellValue(row, currentCol, 0); currentCol++; boardChanged = true; }
                    if (currentCol < BOARD_SIZE - 1 && getCellValue(row, currentCol + 1) == currentValue && !hasMerged[currentCol + 1]) {
                        int newValue = getCellValue(row, currentCol + 1) * 2; setCellValue(row, currentCol + 1, newValue); setCellValue(row, currentCol, 0);
                        currentScore += newValue; hasMerged[currentCol + 1] = true; boardChanged = true;
                    }
                }
            }
        } checkWinCondition(); checkGameOverCondition(); return boardChanged;
    }

    /**
     * Sérialise l'état actuel du jeu (plateau et score courant) en une chaîne.
     * Format: "val,val,...,val,score"
     * @return Chaîne sérialisée.
     */
    @NonNull
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        for (int row = 0; row < BOARD_SIZE; row++) {
            for (int col = 0; col < BOARD_SIZE; col++) { sb.append(board[row][col]).append(","); }
        }
        sb.append(currentScore);
        return sb.toString();
    }

    /**
     * Crée un nouvel objet Game à partir d'une chaîne sérialisée (plateau + score).
     * @param serializedState Chaîne générée par toString().
     * @return Nouvel objet Game, ou null si la désérialisation échoue.
     */
    public static Game deserialize(String serializedState) {
        if (serializedState == null || serializedState.isEmpty()) return null;
        String[] values = serializedState.split(",");
        if (values.length != (BOARD_SIZE * BOARD_SIZE + 1)) return null;
        int[][] newBoard = new int[BOARD_SIZE][BOARD_SIZE]; int index = 0;
        try {
            for (int row = 0; row < BOARD_SIZE; row++) {
                for (int col = 0; col < BOARD_SIZE; col++) { newBoard[row][col] = Integer.parseInt(values[index++]); }
            }
            int score = Integer.parseInt(values[index]);
            return new Game(newBoard, score);
        } catch (NumberFormatException | ArrayIndexOutOfBoundsException e) { return null; }
    }

    /**
     * Vérifie si une tuile >= 2048 existe sur le plateau et met à jour l'état `gameWon`.
     */
    private void checkWinCondition() {
        if (!gameWon) { // Optimisation: inutile de revérifier si déjà gagné
            for(int r=0; r<BOARD_SIZE; r++) for(int c=0; c<BOARD_SIZE; c++) if(getCellValue(r,c)>=2048) { setGameWon(true); return; }
        }
    }

    /**
     * Vérifie s'il reste des mouvements possibles (case vide ou fusion adjacente).
     * Met à jour l'état `gameOver`.
     */
    private void checkGameOverCondition() {
        if (hasEmptyCell()) { setGameOver(false); return; } // Si case vide, pas game over
        // Vérifie fusions adjacentes possibles
        for(int r=0; r<BOARD_SIZE; r++) for(int c=0; c<BOARD_SIZE; c++) {
            int current = getCellValue(r,c);
            // Vérifie voisins (haut, bas, gauche, droite)
            if ((r>0 && getCellValue(r-1,c)==current) || (r<BOARD_SIZE-1 && getCellValue(r+1,c)==current) ||
                    (c>0 && getCellValue(r,c-1)==current) || (c<BOARD_SIZE-1 && getCellValue(r,c+1)==current)) {
                setGameOver(false); return; // Fusion possible, pas game over
            }
        }
        setGameOver(true); // Aucune case vide et aucune fusion -> game over
    }

    /**
     * @return true s'il y a au moins une case vide sur le plateau, false sinon.
     */
    private boolean hasEmptyCell() {
        for(int r=0; r<BOARD_SIZE; r++) for(int c=0; c<BOARD_SIZE; c++) if(getCellValue(r,c)==0) return true;
        return false;
    }

    /**
     * Trouve et retourne la valeur de la tuile la plus élevée actuellement sur le plateau.
     * @return La valeur maximale trouvée.
     */
    public int getHighestTileValue() {
        int maxTile = 0;
        for(int r=0; r<BOARD_SIZE; r++) for(int c=0; c<BOARD_SIZE; c++) if(board[r][c]>maxTile) maxTile=board[r][c];
        return maxTile;
    }
}