// Fichier GameStats.java
// Gère le stockage, le chargement et la mise à jour des statistiques du jeu 2048.
/*
    Fonctions principales :
    - Contient tous les champs relatifs aux statistiques (solo et multijoueur).
    - loadStats(), saveStats() : Charge et sauvegarde les statistiques via SharedPreferences.
    - Méthodes pour mettre à jour les statistiques : startGame(), recordMove(), recordMerge(), recordWin(), recordLoss(), endGame().
    - Getters pour accéder aux valeurs des statistiques.
    - formatTime() : Méthode utilitaire pour formater le temps.

    Relations :
    - MainActivity : Crée une instance de GameStats, l'utilise pour charger/sauvegarder les stats et met à jour les stats via ses méthodes. Récupère les valeurs via les getters pour l'affichage.
    - SharedPreferences : Utilisé pour la persistance des statistiques.
*/
package legion.muyue.best2048;

import android.content.Context;
import android.content.SharedPreferences;
import java.util.concurrent.TimeUnit;

public class GameStats {

    // Clés SharedPreferences (inchangées)
    private static final String PREFS_NAME = "Best2048_Prefs";
    private static final String HIGH_SCORE_KEY = "high_score";
    private static final String STATS_TOTAL_GAMES_PLAYED = "totalGamesPlayed";
    // ... (autres clés inchangées) ...
    private static final String STATS_TOTAL_GAMES_STARTED = "totalGamesStarted";
    private static final String STATS_TOTAL_MOVES = "totalMoves";
    private static final String STATS_TOTAL_PLAY_TIME_MS = "totalPlayTimeMs";
    private static final String STATS_TOTAL_MERGES = "totalMerges";
    private static final String STATS_HIGHEST_TILE = "highestTile";
    private static final String STATS_OBJECTIVE_REACHED_COUNT = "numberOfTimesObjectiveReached";
    private static final String STATS_PERFECT_GAMES = "perfectGames";
    private static final String STATS_BEST_WINNING_TIME_MS = "bestWinningTimeMs";
    private static final String STATS_WORST_WINNING_TIME_MS = "worstWinningTimeMs";
    private static final String STATS_MP_GAMES_WON = "multiplayerGamesWon";
    private static final String STATS_MP_GAMES_PLAYED = "multiplayerGamesPlayed";
    private static final String STATS_MP_BEST_WINNING_STREAK = "multiplayerBestWinningStreak";
    private static final String STATS_MP_TOTAL_SCORE = "multiplayerTotalScore";
    private static final String STATS_MP_TOTAL_TIME_MS = "multiplayerTotalTimeMs";
    private static final String STATS_MP_LOSSES = "totalMultiplayerLosses";
    private static final String STATS_MP_HIGH_SCORE = "multiplayerHighScore";

    // Champs statistiques (inchangés)
    private int totalGamesPlayed;
    private int totalGamesStarted;
    private int totalMoves;
    private int currentMoves;
    private long totalPlayTimeMs;
    private long currentGameStartTimeMs;
    private int mergesThisGame;
    private int totalMerges;
    private int highestTile;
    private int numberOfTimesObjectiveReached;
    private int perfectGames;
    private long bestWinningTimeMs;
    private long worstWinningTimeMs;
    private int multiplayerGamesWon;
    private int multiplayerGamesPlayed;
    private int multiplayerBestWinningStreak;
    private long multiplayerTotalScore;
    private long multiplayerTotalTimeMs;
    private int totalMultiplayerLosses;
    private int multiplayerHighestScore;
    private int overallHighScore;

    private final Context context;

    /**
     * Constructeur de GameStats.
     * Charge immédiatement les statistiques sauvegardées via SharedPreferences.
     * @param context Le contexte de l'application (nécessaire pour SharedPreferences).
     */
    public GameStats(Context context) {
        this.context = context;
        loadStats();
    }

    /**
     * Charge toutes les statistiques (générales et multijoueur) et le high score global
     * depuis les SharedPreferences.
     */
    public void loadStats() {
        SharedPreferences prefs = context.getSharedPreferences(PREFS_NAME, Context.MODE_PRIVATE);
        overallHighScore = prefs.getInt(HIGH_SCORE_KEY, 0);
        totalGamesPlayed = prefs.getInt(STATS_TOTAL_GAMES_PLAYED, 0);
        totalGamesStarted = prefs.getInt(STATS_TOTAL_GAMES_STARTED, 0);
        totalMoves = prefs.getInt(STATS_TOTAL_MOVES, 0);
        totalPlayTimeMs = prefs.getLong(STATS_TOTAL_PLAY_TIME_MS, 0);
        totalMerges = prefs.getInt(STATS_TOTAL_MERGES, 0);
        highestTile = prefs.getInt(STATS_HIGHEST_TILE, 0);
        numberOfTimesObjectiveReached = prefs.getInt(STATS_OBJECTIVE_REACHED_COUNT, 0);
        perfectGames = prefs.getInt(STATS_PERFECT_GAMES, 0);
        bestWinningTimeMs = prefs.getLong(STATS_BEST_WINNING_TIME_MS, Long.MAX_VALUE);
        worstWinningTimeMs = prefs.getLong(STATS_WORST_WINNING_TIME_MS, 0);
        multiplayerGamesWon = prefs.getInt(STATS_MP_GAMES_WON, 0);
        multiplayerGamesPlayed = prefs.getInt(STATS_MP_GAMES_PLAYED, 0);
        multiplayerBestWinningStreak = prefs.getInt(STATS_MP_BEST_WINNING_STREAK, 0);
        multiplayerTotalScore = prefs.getLong(STATS_MP_TOTAL_SCORE, 0);
        multiplayerTotalTimeMs = prefs.getLong(STATS_MP_TOTAL_TIME_MS, 0);
        totalMultiplayerLosses = prefs.getInt(STATS_MP_LOSSES, 0);
        multiplayerHighestScore = prefs.getInt(STATS_MP_HIGH_SCORE, 0);
    }

    /**
     * Sauvegarde toutes les statistiques (générales et multijoueur) et le high score global
     * dans les SharedPreferences.
     */
    public void saveStats() {
        SharedPreferences prefs = context.getSharedPreferences(PREFS_NAME, Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = prefs.edit();
        editor.putInt(HIGH_SCORE_KEY, overallHighScore);
        editor.putInt(STATS_TOTAL_GAMES_PLAYED, totalGamesPlayed);
        editor.putInt(STATS_TOTAL_GAMES_STARTED, totalGamesStarted);
        editor.putInt(STATS_TOTAL_MOVES, totalMoves);
        editor.putLong(STATS_TOTAL_PLAY_TIME_MS, totalPlayTimeMs);
        editor.putInt(STATS_TOTAL_MERGES, totalMerges);
        editor.putInt(STATS_HIGHEST_TILE, highestTile);
        editor.putInt(STATS_OBJECTIVE_REACHED_COUNT, numberOfTimesObjectiveReached);
        editor.putInt(STATS_PERFECT_GAMES, perfectGames);
        editor.putLong(STATS_BEST_WINNING_TIME_MS, bestWinningTimeMs);
        editor.putLong(STATS_WORST_WINNING_TIME_MS, worstWinningTimeMs);
        editor.putInt(STATS_MP_GAMES_WON, multiplayerGamesWon);
        editor.putInt(STATS_MP_GAMES_PLAYED, multiplayerGamesPlayed);
        editor.putInt(STATS_MP_BEST_WINNING_STREAK, multiplayerBestWinningStreak);
        editor.putLong(STATS_MP_TOTAL_SCORE, multiplayerTotalScore);
        editor.putLong(STATS_MP_TOTAL_TIME_MS, multiplayerTotalTimeMs);
        editor.putInt(STATS_MP_LOSSES, totalMultiplayerLosses);
        editor.putInt(STATS_MP_HIGH_SCORE, multiplayerHighestScore);
        editor.apply();
    }

    /**
     * Met à jour les statistiques lors du démarrage d'une nouvelle partie.
     * Incrémente le nombre de parties démarrées et réinitialise les compteurs de la partie en cours.
     */
    public void startGame() {
        totalGamesStarted++;
        currentMoves = 0;
        mergesThisGame = 0;
        currentGameStartTimeMs = System.currentTimeMillis();
    }

    /**
     * Enregistre un mouvement effectué pendant la partie en cours.
     * Incrémente le compteur de mouvements de la partie et le compteur total.
     */
    public void recordMove() {
        currentMoves++;
        totalMoves++;
    }

    /**
     * Enregistre une ou plusieurs fusions survenues pendant la partie en cours.
     * @param numberOfMerges Le nombre de fusions à ajouter (idéalement précis).
     */
    public void recordMerge(int numberOfMerges) {
        if (numberOfMerges > 0) {
            mergesThisGame += numberOfMerges;
            totalMerges += numberOfMerges;
        }
    }

    /**
     * Met à jour la statistique de la plus haute tuile atteinte si la valeur fournie est supérieure.
     * @param tileValue La valeur de la tuile candidate.
     */
    public void updateHighestTile(int tileValue) {
        if (tileValue > this.highestTile) {
            this.highestTile = tileValue;
        }
    }

    /**
     * Enregistre une victoire et met à jour les statistiques associées (temps, nombre de victoires).
     * @param timeTakenMs Le temps mis pour gagner cette partie en millisecondes.
     */
    public void recordWin(long timeTakenMs) {
        numberOfTimesObjectiveReached++;
        endGame(timeTakenMs); // Finalise les stats de la partie

        if (timeTakenMs < bestWinningTimeMs) { bestWinningTimeMs = timeTakenMs; }
        if (timeTakenMs > worstWinningTimeMs) { worstWinningTimeMs = timeTakenMs; }
    }

    /**
     * Enregistre une défaite et finalise les statistiques de la partie.
     */
    public void recordLoss() {
        endGame(System.currentTimeMillis() - currentGameStartTimeMs);
    }

    /**
     * Finalise les statistiques à la fin d'une partie (incrémente parties jouées, ajoute temps de jeu).
     * @param timeTakenMs Le temps total de la partie qui vient de se terminer.
     */
    public void endGame(long timeTakenMs) {
        totalGamesPlayed++;
        addPlayTime(timeTakenMs);
    }

    /**
     * Ajoute une durée au temps de jeu total cumulé.
     * @param durationMs Durée à ajouter en millisecondes.
     */
    public void addPlayTime(long durationMs) {
        if (durationMs > 0) {
            this.totalPlayTimeMs += durationMs;
        }
    }

    // --- Getters ---
    public int getTotalGamesPlayed() { return totalGamesPlayed; }
    public int getTotalGamesStarted() { return totalGamesStarted; }
    public int getTotalMoves() { return totalMoves; }
    public int getCurrentMoves() { return currentMoves; }
    public long getTotalPlayTimeMs() { return totalPlayTimeMs; }
    public long getCurrentGameStartTimeMs() { return currentGameStartTimeMs; }
    public int getMergesThisGame() { return mergesThisGame; }
    public int getTotalMerges() { return totalMerges; }
    public int getHighestTile() { return highestTile; }
    public int getNumberOfTimesObjectiveReached() { return numberOfTimesObjectiveReached; }
    public int getPerfectGames() { return perfectGames; }
    public long getBestWinningTimeMs() { return bestWinningTimeMs; }
    public long getWorstWinningTimeMs() { return worstWinningTimeMs; }
    public int getMultiplayerGamesWon() { return multiplayerGamesWon; }
    public int getMultiplayerGamesPlayed() { return multiplayerGamesPlayed; }
    public int getMultiplayerBestWinningStreak() { return multiplayerBestWinningStreak; }
    public long getMultiplayerTotalScore() { return multiplayerTotalScore; }
    public long getMultiplayerTotalTimeMs() { return multiplayerTotalTimeMs; }
    public int getTotalMultiplayerLosses() { return totalMultiplayerLosses; }
    public int getMultiplayerHighestScore() { return multiplayerHighestScore; }
    public int getOverallHighScore() { return overallHighScore; } // Getter pour le HS global

    // --- Setters ---
    /**
     * Met à jour la valeur interne du meilleur score global.
     * Appelé par MainActivity pour synchroniser le high score lu depuis les préférences.
     * @param highScore Le meilleur score lu.
     */
    public void setHighestScore(int highScore) {
        // On met à jour seulement si la valeur externe est supérieure,
        // car GameStats gère aussi la mise à jour via les scores des parties.
        // Ou plus simplement, on fait confiance à la valeur lue par MainActivity.
        this.overallHighScore = highScore;
    }

    /**
     * Définit le timestamp de démarrage pour la partie en cours.
     * @param timeMs Timestamp en millisecondes.
     */
    public void setCurrentGameStartTimeMs(long timeMs) {
        this.currentGameStartTimeMs = timeMs;
    }

    // --- Méthodes calculées ---
    public long getAverageGameTimeMs() { return (totalGamesPlayed > 0) ? totalPlayTimeMs / totalGamesPlayed : 0; }
    public int getMultiplayerAverageScore() { return (multiplayerGamesPlayed > 0) ? (int)(multiplayerTotalScore / multiplayerGamesPlayed) : 0; }
    public long getMultiplayerAverageTimeMs() { return (multiplayerGamesPlayed > 0) ? multiplayerTotalTimeMs / multiplayerGamesPlayed : 0; }

    /**
     * Formate une durée en millisecondes en chaîne "hh:mm:ss" ou "mm:ss".
     * @param milliseconds Durée en millisecondes.
     * @return Chaîne de temps formatée.
     */
    public static String formatTime(long milliseconds) {
        long hours = TimeUnit.MILLISECONDS.toHours(milliseconds);
        long minutes = TimeUnit.MILLISECONDS.toMinutes(milliseconds) % 60;
        long seconds = TimeUnit.MILLISECONDS.toSeconds(milliseconds) % 60;
        if (hours > 0) { return String.format("%02d:%02d:%02d", hours, minutes, seconds); }
        else { return String.format("%02d:%02d", minutes, seconds); }
    }
}