// Fichier MainActivity.java
// Activité principale de l'application 2048, gère l'interface utilisateur et la coordination du jeu.
/*
    Fonctions principales :
    - findViews() : Récupère les références des éléments UI du layout XML.
    - initializeGameAndStats() : Crée les instances de Game et GameStats, charge les données sauvegardées.
    - setupListeners() : Attache les listeners aux boutons et au plateau de jeu (pour les swipes).
    - updateUI(), updateBoard(), updateScores() : Met à jour l'affichage du jeu (plateau, scores).
    - setTileStyle() : Applique le style visuel à une tuile individuelle.
    - handleSwipe() : Traite un geste de swipe, appelle la logique de Game, met à jour les stats et l'UI.
    - startNewGame() : Initialise une nouvelle partie.
    - showRestartConfirmationDialog(), showGameWonDialog(), showGameOverDialog(), showMenu(), showMultiplayerScreen() : Affiche diverses boîtes de dialogue.
    - toggleStatistics(), updateStatisticsTextViews() : Gère l'affichage et la mise à jour du panneau de statistiques (via ViewStub).
    - Cycle de vie : onCreate(), onResume(), onPause() pour initialiser, reprendre le timer, et sauvegarder l'état/stats.
    - Persistance : Utilise SharedPreferences pour sauvegarder/charger l'état du jeu (via Game.toString/deserialize) et le meilleur score. Les stats détaillées sont gérées par GameStats.

    Relations :
    - Game : Instance contenant la logique pure du jeu (plateau, score, mouvements).
    - GameStats : Instance contenant la logique et les données des statistiques.
    - OnSwipeTouchListener : Détecte les swipes sur le plateau.
    - Layout XML (activity_main.xml, stats_layout.xml, etc.) : Définit la structure de l'UI.
    - Ressources (strings, colors, dimens, styles, drawables) : Utilisées pour l'apparence de l'UI.
    - SharedPreferences : Pour la sauvegarde des données persistantes.
*/
package legion.muyue.best2048;

import android.annotation.SuppressLint;
import android.app.AlertDialog;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.util.TypedValue;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewStub;
import android.view.animation.AnimationUtils;
import android.widget.TextView;

import androidx.activity.EdgeToEdge;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.content.ContextCompat;
import androidx.gridlayout.widget.GridLayout;
import android.widget.Button;


public class MainActivity extends AppCompatActivity {

    // --- UI Elements ---
    private GridLayout boardGridLayout;
    private TextView currentScoreTextView;
    private TextView highestScoreTextView;
    private Button newGameButton;
    private Button multiplayerButton;
    private Button statisticsButton;
    private Button menuButton;
    private ViewStub statisticsViewStub;
    private View inflatedStatsView; // Référence à la vue des stats une fois gonflée

    // --- Game Logic & Stats ---
    private Game game;
    private GameStats gameStats; // Instance pour gérer les stats
    private static final int BOARD_SIZE = 4;

    // --- Preferences ---
    private SharedPreferences preferences;
    private static final String PREFS_NAME = "Best2048_Prefs";
    private static final String HIGH_SCORE_KEY = "high_score";
    private static final String GAME_STATE_KEY = "game_state";

    private boolean statisticsVisible = false;

    // --- Activity Lifecycle ---

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        EdgeToEdge.enable(this);
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        findViews(); // Récupère les vues
        initializeGameAndStats(); // Initialise Game, GameStats et charge les données
        setupListeners(); // Attache les listeners
    }

    @Override
    protected void onResume() {
        super.onResume();
        // Redémarre le chrono pour la partie en cours SI elle n'est pas finie
        if (game != null && gameStats != null && !game.isGameOver() && !game.isGameWon()) {
            gameStats.setCurrentGameStartTimeMs(System.currentTimeMillis()); // Utilise setter de GameStats
        }
        // Gère le réaffichage potentiel des stats si l'activité reprend
        if (statisticsVisible) {
            if (inflatedStatsView != null) { // Si déjà gonflé
                updateStatisticsTextViews(); // Met à jour les données affichées
                inflatedStatsView.setVisibility(View.VISIBLE);
                multiplayerButton.setVisibility(View.GONE);
            } else {
                // Si pas encore gonflé (cas rare mais possible), on le fait afficher
                toggleStatistics();
            }
        }
    }

    @Override
    protected void onPause() {
        super.onPause();
        // Sauvegarde l'état et les stats si le jeu existe
        if (game != null && gameStats != null) {
            // Met à jour le temps total SI la partie n'est pas terminée
            if (!game.isGameOver() && !game.isGameWon()) {
                gameStats.addPlayTime(System.currentTimeMillis() - gameStats.getCurrentGameStartTimeMs()); // Utilise méthode GameStats
            }
            saveGame();       // Sauvegarde l'état du jeu (plateau + score courant) et le HS
            gameStats.saveStats(); // Sauvegarde toutes les stats via GameStats
        }
    }

    // --- Initialisation ---

    /**
     * Récupère les références des vues du layout principal via leur ID.
     */
    private void findViews() {
        boardGridLayout = findViewById(R.id.gameBoard);
        currentScoreTextView = findViewById(R.id.scoreLabel);
        highestScoreTextView = findViewById(R.id.highScoreLabel);
        newGameButton = findViewById(R.id.restartButton);
        statisticsButton = findViewById(R.id.statsButton);
        menuButton = findViewById(R.id.menuButton);
        multiplayerButton = findViewById(R.id.multiplayerButton);
        statisticsViewStub = findViewById(R.id.statsViewStub);
    }

    /**
     * Initialise les objets Game et GameStats.
     * Charge l'état du jeu sauvegardé (s'il existe) et le meilleur score.
     * Met à jour l'interface utilisateur initiale.
     */
    private void initializeGameAndStats() {
        preferences = getSharedPreferences(PREFS_NAME, MODE_PRIVATE);
        gameStats = new GameStats(this); // Crée et charge les stats (y.c. overallHighScore)
        loadGame(); // Charge l'état du jeu, crée un nouveau si nécessaire, synchronise le HS
        updateUI(); // Affiche l'état chargé ou nouveau

        // Si loadGame a résulté en une nouvelle partie (game==null avant ou deserialize a échoué),
        // on s'assure que les stats de la partie en cours sont bien initialisées.
        if (game == null || preferences.getString(GAME_STATE_KEY, null) == null) {
            startNewGame(); // Assure un état cohérent si aucun jeu n'a été chargé
        } else {
            // Le timer de la partie chargée sera (re)démarré dans onResume
        }
    }

    /**
     * Attache les listeners aux boutons et configure le listener de swipe sur le plateau.
     */
    private void setupListeners() {
        newGameButton.setOnClickListener(v -> {
            v.startAnimation(AnimationUtils.loadAnimation(this, R.anim.button_press));
            showRestartConfirmationDialog();
        });
        statisticsButton.setOnClickListener(v -> {
            v.startAnimation(AnimationUtils.loadAnimation(this, R.anim.button_press));
            toggleStatistics(); // Affiche/masque les stats
        });
        menuButton.setOnClickListener(v -> {
            v.startAnimation(AnimationUtils.loadAnimation(this, R.anim.button_press));
            showMenu(); // Affiche dialogue placeholder
        });
        multiplayerButton.setOnClickListener(v -> {
            v.startAnimation(AnimationUtils.loadAnimation(this, R.anim.button_press));
            showMultiplayerScreen(); // Affiche dialogue placeholder
        });
        setupSwipeListener(); // Attache le listener de swipe
    }

    // --- Mise à jour UI ---

    /**
     * Met à jour complètement l'interface utilisateur (plateau et scores).
     */
    private void updateUI() {
        if (game == null) return;
        updateBoard();
        updateScores();
    }

    /**
     * Redessine le plateau de jeu en créant/mettant à jour les TextViews des tuiles.
     */
    private void updateBoard() {
        boardGridLayout.removeAllViews();
        for (int row = 0; row < BOARD_SIZE; row++) {
            for (int col = 0; col < BOARD_SIZE; col++) {
                TextView tileTextView = new TextView(this);
                int value = game.getCellValue(row, col);
                setTileStyle(tileTextView, value); // Applique le style
                // Définit les LayoutParams pour que la tuile remplisse la cellule du GridLayout
                GridLayout.LayoutParams params = new GridLayout.LayoutParams();
                params.width = 0; params.height = 0; // Poids gère la taille
                params.rowSpec = GridLayout.spec(row, 1f); // Prend 1 fraction de l'espace en hauteur
                params.columnSpec = GridLayout.spec(col, 1f); // Prend 1 fraction de l'espace en largeur
                int margin = (int) getResources().getDimension(R.dimen.tile_margin);
                params.setMargins(margin, margin, margin, margin); // Applique les marges
                tileTextView.setLayoutParams(params);
                boardGridLayout.addView(tileTextView); // Ajoute la tuile au GridLayout
            }
        }
    }

    /**
     * Met à jour les TextViews affichant le score courant et le meilleur score.
     */
    private void updateScores() {
        currentScoreTextView.setText(getString(R.string.score_placeholder, game.getCurrentScore()));
        highestScoreTextView.setText(getString(R.string.high_score_placeholder, game.getHighestScore()));
    }

    /**
     * Applique le style visuel (fond, texte, taille) à une TextView représentant une tuile.
     * @param tileTextView La TextView de la tuile.
     * @param value La valeur numérique de la tuile (0 pour vide).
     */
    private void setTileStyle(TextView tileTextView, int value) {
        // Code de styling (inchangé par rapport à la correction précédente)
        tileTextView.setText(value > 0 ? String.valueOf(value) : "");
        tileTextView.setGravity(Gravity.CENTER);
        tileTextView.setTypeface(null, android.graphics.Typeface.BOLD);
        int backgroundColorId; int textColorId; int textSizeId;
        switch (value) {
            case 0:    backgroundColorId = R.color.tile_empty; textColorId = android.R.color.transparent; textSizeId = R.dimen.text_size_tile_small; break;
            case 2:    backgroundColorId = R.color.tile_2;     textColorId = R.color.text_tile_low;     textSizeId = R.dimen.text_size_tile_small; break;
            case 4:    backgroundColorId = R.color.tile_4;     textColorId = R.color.text_tile_low;     textSizeId = R.dimen.text_size_tile_small; break;
            case 8:    backgroundColorId = R.color.tile_8;     textColorId = R.color.text_tile_high;    textSizeId = R.dimen.text_size_tile_small; break;
            case 16:   backgroundColorId = R.color.tile_16;    textColorId = R.color.text_tile_high;    textSizeId = R.dimen.text_size_tile_small; break;
            case 32:   backgroundColorId = R.color.tile_32;    textColorId = R.color.text_tile_high;    textSizeId = R.dimen.text_size_tile_small; break;
            case 64:   backgroundColorId = R.color.tile_64;    textColorId = R.color.text_tile_high;    textSizeId = R.dimen.text_size_tile_small; break;
            case 128:  backgroundColorId = R.color.tile_128;   textColorId = R.color.text_tile_high;    textSizeId = R.dimen.text_size_tile_medium; break;
            case 256:  backgroundColorId = R.color.tile_256;   textColorId = R.color.text_tile_high;    textSizeId = R.dimen.text_size_tile_medium; break;
            case 512:  backgroundColorId = R.color.tile_512;   textColorId = R.color.text_tile_high;    textSizeId = R.dimen.text_size_tile_medium; break;
            case 1024: backgroundColorId = R.color.tile_1024;  textColorId = R.color.text_tile_high;    textSizeId = R.dimen.text_size_tile_large; break;
            case 2048: backgroundColorId = R.color.tile_2048;  textColorId = R.color.text_tile_high;    textSizeId = R.dimen.text_size_tile_large; break;
            default:   backgroundColorId = R.color.tile_super;   textColorId = R.color.text_tile_high;    textSizeId = R.dimen.text_size_tile_large; break;
        }
        tileTextView.setBackgroundResource(R.drawable.tile_background);
        tileTextView.getBackground().setTint(ContextCompat.getColor(this, backgroundColorId));
        tileTextView.setTextColor(ContextCompat.getColor(this, textColorId));
        tileTextView.setTextSize(TypedValue.COMPLEX_UNIT_PX, getResources().getDimension(textSizeId));
    }


    // --- Gestion des Actions Utilisateur ---

    /**
     * Configure le listener pour détecter les swipes sur le plateau de jeu.
     */
    @SuppressLint("ClickableViewAccessibility")
    private void setupSwipeListener() {
        boardGridLayout.setOnTouchListener(new OnSwipeTouchListener(this, new OnSwipeTouchListener.SwipeListener() {
            @Override public void onSwipeTop() { handleSwipe(Direction.UP); }
            @Override public void onSwipeBottom() { handleSwipe(Direction.DOWN); }
            @Override public void onSwipeLeft() { handleSwipe(Direction.LEFT); }
            @Override public void onSwipeRight() { handleSwipe(Direction.RIGHT); }
        }));
    }

    /**
     * Traite un geste de swipe : appelle la logique de jeu, met à jour les statistiques,
     * ajoute une nouvelle tuile, met à jour l'UI et vérifie les conditions de fin de partie.
     * @param direction Direction du swipe.
     */
    private void handleSwipe(Direction direction) {
        if (game == null || gameStats == null || game.isGameOver() || game.isGameWon()) return;

        int scoreBefore = game.getCurrentScore();
        boolean boardChanged;
        switch (direction) { /* ... appelle game.pushX() ... */
            case UP:    boardChanged = game.pushUp();    break;
            case DOWN:  boardChanged = game.pushDown();  break;
            case LEFT:  boardChanged = game.pushLeft();  break;
            case RIGHT: boardChanged = game.pushRight(); break;
            default:    boardChanged = false;
        }

        if (boardChanged) {
            gameStats.recordMove(); // Met à jour stats mouvement
            int scoreAfter = game.getCurrentScore();
            int scoreDelta = scoreAfter - scoreBefore;
            if (scoreDelta > 0) {
                gameStats.recordMerge(1); // Met à jour stats fusion (simplifié)
                // Met à jour le highScore dans Game et GameStats si nécessaire
                if (scoreAfter > game.getHighestScore()) {
                    game.setHighestScore(scoreAfter);
                    gameStats.setHighestScore(scoreAfter); // Synchronise avec GameStats
                }
            }

            gameStats.updateHighestTile(game.getHighestTileValue()); // Met à jour stat tuile max

            game.addNewTile(); // Ajoute tuile
            updateUI(); // Met à jour affichage

            // Vérifie victoire/défaite après MAJ UI
            if (game.isGameWon()) {
                long timeTaken = System.currentTimeMillis() - gameStats.getCurrentGameStartTimeMs();
                gameStats.recordWin(timeTaken);
                showGameWonDialog();
            } else if (game.isGameOver()) {
                long timeTaken = System.currentTimeMillis() - gameStats.getCurrentGameStartTimeMs();
                gameStats.recordLoss(); // Appelle endGame implicitement
                showGameOverDialog();
            }
        }
    }

    // Enum Direction (inchangé)
    enum Direction { UP, DOWN, LEFT, RIGHT }

    /**
     * Affiche la boîte de dialogue demandant confirmation avant de redémarrer.
     */
    private void showRestartConfirmationDialog() {
        // ... (code de la dialog inchangé, appelle startNewGame) ...
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        LayoutInflater inflater = getLayoutInflater();
        View dialogView = inflater.inflate(R.layout.dialog_restart_confirm, null);
        builder.setView(dialogView);
        Button cancelButton = dialogView.findViewById(R.id.dialogCancelButton);
        Button confirmButton = dialogView.findViewById(R.id.dialogConfirmButton);
        final AlertDialog dialog = builder.create();
        cancelButton.setOnClickListener(v -> dialog.dismiss());
        confirmButton.setOnClickListener(v -> { dialog.dismiss(); startNewGame(); });
        dialog.show();
    }

    /**
     * Démarre une nouvelle partie : initialise GameStats pour une nouvelle session,
     * crée un nouvel objet Game, synchronise le meilleur score et met à jour l'UI.
     */
    private void startNewGame() {
        gameStats.startGame(); // Réinitialise stats partie en cours (mouvements, temps, etc.)
        game = new Game();     // Crée un nouveau jeu logique
        // Applique le meilleur score global connu (chargé par gameStats) au nouvel objet Game
        game.setHighestScore(gameStats.getOverallHighScore());
        updateUI();            // Rafraîchit l'affichage
    }


    // --- Gestion des Statistiques (UI) ---

    /**
     * Affiche ou masque le panneau de statistiques.
     * Gonfle le layout via ViewStub si c'est la première fois.
     */
    private void toggleStatistics() {
        statisticsVisible = !statisticsVisible;
        if (statisticsVisible) {
            if (inflatedStatsView == null) { // Gonfle si pas encore fait
                inflatedStatsView = statisticsViewStub.inflate();
                // Attache listener au bouton Back une fois la vue gonflée
                Button backButton = inflatedStatsView.findViewById(R.id.backButton);
                backButton.setOnClickListener(v -> toggleStatistics()); // Recliquer sur Back re-appelle toggle
            }
            updateStatisticsTextViews(); // Remplit les champs avec les données actuelles
            inflatedStatsView.setVisibility(View.VISIBLE); // Affiche
            multiplayerButton.setVisibility(View.GONE); // Masque bouton multi
        } else {
            if (inflatedStatsView != null) { // Masque si la vue existe
                inflatedStatsView.setVisibility(View.GONE);
            }
            multiplayerButton.setVisibility(View.VISIBLE); // Réaffiche bouton multi
        }
    }

    /**
     * Remplit les TextViews du panneau de statistiques avec les données de l'objet GameStats.
     * Doit être appelé seulement après que `inflatedStatsView` a été initialisé.
     */
    private void updateStatisticsTextViews() {
        if (inflatedStatsView == null || gameStats == null) return;

        // Récupération des TextViews dans la vue gonflée (idem)
        TextView highScoreStatsLabel = inflatedStatsView.findViewById(R.id.high_score_stats_label);
        TextView totalGamesPlayedLabel = inflatedStatsView.findViewById(R.id.total_games_played_label);
        TextView totalGamesStartedLabel = inflatedStatsView.findViewById(R.id.total_games_started_label);
        TextView winPercentageLabel = inflatedStatsView.findViewById(R.id.win_percentage_label);
        TextView totalPlayTimeLabel = inflatedStatsView.findViewById(R.id.total_play_time_label);
        TextView totalMovesLabel = inflatedStatsView.findViewById(R.id.total_moves_label);
        TextView currentMovesLabel = inflatedStatsView.findViewById(R.id.current_moves_label);
        TextView currentGameTimeLabel = inflatedStatsView.findViewById(R.id.current_game_time_label);
        TextView averageGameTimeLabel = inflatedStatsView.findViewById(R.id.average_game_time_label);
        TextView bestWinningTimeLabel = inflatedStatsView.findViewById(R.id.best_winning_time_label);
        TextView worstWinningTimeLabel = inflatedStatsView.findViewById(R.id.worst_winning_time_label);
        TextView totalMergesLabel = inflatedStatsView.findViewById(R.id.total_merges_label);
        TextView highestTileLabel = inflatedStatsView.findViewById(R.id.highest_tile_label);
        TextView objectiveReachedLabel = inflatedStatsView.findViewById(R.id.number_of_time_objective_reached_label);
        TextView perfectGameLabel = inflatedStatsView.findViewById(R.id.perfect_game_label);
        TextView multiplayerGamesWonLabel = inflatedStatsView.findViewById(R.id.multiplayer_games_won_label);
        TextView multiplayerGamesPlayedLabel = inflatedStatsView.findViewById(R.id.multiplayer_games_played_label);
        TextView multiplayerWinRateLabel = inflatedStatsView.findViewById(R.id.multiplayer_win_rate_label);
        TextView multiplayerBestWinningStreakLabel = inflatedStatsView.findViewById(R.id.multiplayer_best_winning_streak_label);
        TextView multiplayerAverageScoreLabel = inflatedStatsView.findViewById(R.id.multiplayer_average_score_label);
        TextView averageTimePerGameMultiLabel = inflatedStatsView.findViewById(R.id.average_time_per_game_label); // ID toujours potentiellement dupliqué
        TextView totalMultiplayerLossesLabel = inflatedStatsView.findViewById(R.id.total_multiplayer_losses_label);
        TextView multiplayerHighScoreLabel = inflatedStatsView.findViewById(R.id.multiplayer_high_score_label);
        TextView mergesThisGameLabel = inflatedStatsView.findViewById(R.id.merges_this_game);

        // MAJ textes avec getters de gameStats
        highScoreStatsLabel.setText(getString(R.string.high_score_stats, gameStats.getOverallHighScore()));
        totalGamesPlayedLabel.setText(getString(R.string.total_games_played, gameStats.getTotalGamesPlayed()));
        totalGamesStartedLabel.setText(getString(R.string.total_games_started, gameStats.getTotalGamesStarted()));
        totalMovesLabel.setText(getString(R.string.total_moves, gameStats.getTotalMoves()));
        currentMovesLabel.setText(getString(R.string.current_moves, gameStats.getCurrentMoves()));
        mergesThisGameLabel.setText(getString(R.string.merges_this_game_label, gameStats.getMergesThisGame()));
        totalMergesLabel.setText(getString(R.string.total_merges, gameStats.getTotalMerges()));
        highestTileLabel.setText(getString(R.string.highest_tile, gameStats.getHighestTile()));
        objectiveReachedLabel.setText(getString(R.string.number_of_time_objective_reached, gameStats.getNumberOfTimesObjectiveReached()));
        perfectGameLabel.setText(getString(R.string.perfect_games, gameStats.getPerfectGames()));
        multiplayerGamesWonLabel.setText(getString(R.string.multiplayer_games_won, gameStats.getMultiplayerGamesWon()));
        multiplayerGamesPlayedLabel.setText(getString(R.string.multiplayer_games_played, gameStats.getMultiplayerGamesPlayed()));
        multiplayerBestWinningStreakLabel.setText(getString(R.string.multiplayer_best_winning_streak, gameStats.getMultiplayerBestWinningStreak()));
        multiplayerAverageScoreLabel.setText(getString(R.string.multiplayer_average_score, gameStats.getMultiplayerAverageScore()));
        totalMultiplayerLossesLabel.setText(getString(R.string.total_multiplayer_losses, gameStats.getTotalMultiplayerLosses()));
        multiplayerHighScoreLabel.setText(getString(R.string.multiplayer_high_score, gameStats.getMultiplayerHighestScore()));

        // Calculs Pourcentages
        String winPercentage = (gameStats.getTotalGamesStarted() > 0) ? String.format("%.2f%%", ((double) gameStats.getNumberOfTimesObjectiveReached() / gameStats.getTotalGamesStarted()) * 100) : "N/A";
        winPercentageLabel.setText(getString(R.string.win_percentage, winPercentage));
        String multiplayerWinRate = (gameStats.getMultiplayerGamesPlayed() > 0) ? String.format("%.2f%%", ((double) gameStats.getMultiplayerGamesWon() / gameStats.getMultiplayerGamesPlayed()) * 100) : "N/A";
        multiplayerWinRateLabel.setText(getString(R.string.multiplayer_win_rate, multiplayerWinRate));

        // Calculs Temps
        totalPlayTimeLabel.setText(getString(R.string.total_play_time, GameStats.formatTime(gameStats.getTotalPlayTimeMs())));
        long currentDuration = (game != null && !game.isGameOver() && !game.isGameWon() && gameStats != null) ? System.currentTimeMillis() - gameStats.getCurrentGameStartTimeMs() : 0;
        currentGameTimeLabel.setText(getString(R.string.current_game_time, GameStats.formatTime(currentDuration)));
        averageGameTimeLabel.setText(getString(R.string.average_time_per_game, GameStats.formatTime(gameStats.getAverageGameTimeMs())));
        averageTimePerGameMultiLabel.setText(getString(R.string.average_time_per_game_label, GameStats.formatTime(gameStats.getMultiplayerAverageTimeMs())));
        bestWinningTimeLabel.setText(getString(R.string.best_winning_time, (gameStats.getBestWinningTimeMs() != Long.MAX_VALUE) ? GameStats.formatTime(gameStats.getBestWinningTimeMs()) : "N/A"));
        worstWinningTimeLabel.setText(getString(R.string.worst_winning_time, (gameStats.getWorstWinningTimeMs() != 0) ? GameStats.formatTime(gameStats.getWorstWinningTimeMs()) : "N/A"));
    }

    // --- Dialogues / Placeholders ---

    /** Affiche un dialogue placeholder pour le menu. */
    private void showMenu() { /* ... (inchangé) ... */
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setTitle("Menu").setMessage("Fonctionnalité de menu à venir !").setPositiveButton("OK", null);
        builder.create().show();
    }

    /** Affiche un dialogue placeholder pour le multijoueur. */
    private void showMultiplayerScreen() { /* ... (inchangé) ... */
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setTitle("Multijoueur").setMessage("Fonctionnalité multijoueur à venir !").setPositiveButton("OK", null);
        builder.create().show();
    }

    /** Affiche le dialogue de victoire. */
    private void showGameWonDialog() { /* ... (inchangé) ... */
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setTitle("Vous avez gagné !")
                .setMessage("Félicitations ! Vous avez atteint 2048 !")
                .setPositiveButton("Nouvelle partie", (dialog, which) -> startNewGame())
                .setNegativeButton("Quitter", (dialog, which) -> finish())
                .setCancelable(false)
                .show();
    }

    /** Affiche le dialogue de défaite. */
    private void showGameOverDialog() { /* ... (inchangé) ... */
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setTitle("Partie terminée !")
                .setMessage("Aucun mouvement possible. Votre score final est : " + game.getCurrentScore())
                .setPositiveButton("Nouvelle partie", (dialog, which) -> startNewGame())
                .setNegativeButton("Quitter", (dialog, which) -> finish())
                .setCancelable(false)
                .show();
    }

    // --- Sauvegarde / Chargement ---

    /** Sauvegarde l'état du jeu et le meilleur score via SharedPreferences. */
    private void saveGame() {
        SharedPreferences.Editor editor = preferences.edit();
        if (game != null) {
            editor.putString(GAME_STATE_KEY, game.toString()); // Sérialise Game (plateau + score courant)
            editor.putInt(HIGH_SCORE_KEY, game.getHighestScore()); // Sauvegarde le HS contenu dans Game
        } else {
            editor.remove(GAME_STATE_KEY); // Optionnel: nettoyer si pas de jeu
        }
        editor.apply();
    }

    /** Charge l'état du jeu depuis SharedPreferences et synchronise le meilleur score. */
    private void loadGame() {
        String gameState = preferences.getString(GAME_STATE_KEY, null);
        int savedHighScore = preferences.getInt(HIGH_SCORE_KEY, gameStats.getOverallHighScore()); // Utilise HS de GameStats comme défaut

        // S'assure que GameStats a la dernière version connue du HS
        gameStats.setHighestScore(savedHighScore);

        if (gameState != null) {
            game = Game.deserialize(gameState); // Désérialise plateau + score
            if (game != null) {
                game.setHighestScore(savedHighScore); // Applique le HS à l'objet Game
            } else {
                // Échec -> Nouvelle partie
                game = new Game();
                game.setHighestScore(savedHighScore);
                gameStats.startGame(); // Réinitialise stats de partie
            }
        } else {
            // Pas de sauvegarde -> Nouvelle partie
            game = new Game();
            game.setHighestScore(savedHighScore);
            gameStats.startGame(); // Réinitialise stats de partie
        }
    }

}