// Fichier OnSwipeTouchListener.java
// Classe utilitaire pour détecter les gestes de balayage (swipe).
/*
    Fonctions principales :
    - Utilise GestureDetector pour détecter les swipes.
    - Définit une interface SwipeListener pour notifier la classe appelante (MainActivity).
    - onFling() : Détecte la direction du swipe (haut, bas, gauche, droite).
    - SWIPE_THRESHOLD, SWIPE_VELOCITY_THRESHOLD : Constantes pour la sensibilité du swipe.

    Relations :
    - MainActivity : MainActivity crée une instance et est notifiée des swipes via l'interface SwipeListener.
*/

package legion.muyue.best2048;

import android.annotation.SuppressLint; // Ajout
import android.content.Context;
// import android.location.GnssAntennaInfo; // Import inutile trouvé dans le dump
import android.view.GestureDetector;
import android.view.MotionEvent;
import android.view.View;

import androidx.annotation.NonNull; // Ajout

public class OnSwipeTouchListener implements View.OnTouchListener {

    private final GestureDetector gestureDetector;
    private final SwipeListener listener;

    /**
     * Interface pour les événements de swipe.  Toute classe qui veut réagir aux swipes
     * doit implémenter cette interface et passer une instance à OnSwipeTouchListener.
     */
    public interface SwipeListener {
        /**
         * Appelée lorsqu'un swipe vers le haut est détecté.
         */
        void onSwipeTop();

        /**
         * Appelée lorsqu'un swipe vers le bas est détecté.
         */
        void onSwipeBottom();

        /**
         * Appelée lorsqu'un swipe vers la gauche est détecté.
         */
        void onSwipeLeft();

        /**
         * Appelée lorsqu'un swipe vers la droite est détecté.
         */
        void onSwipeRight();
    }

    /**
     * Constructeur de la classe OnSwipeTouchListener.
     *
     * @param ctx Le contexte de l'application.  Nécessaire pour GestureDetector.
     * @param listener L'instance qui écoutera les événements de swipe.
     */
    public OnSwipeTouchListener(Context ctx, SwipeListener listener) {
        gestureDetector = new GestureDetector(ctx, new GestureListener());
        this.listener = listener;
    }

    /**
     * Méthode appelée lorsqu'un événement tactile se produit sur la vue attachée.
     * Elle transmet l'événement au GestureDetector pour analyse.
     *
     * @param v     La vue sur laquelle l'événement tactile s'est produit.
     * @param event L'objet MotionEvent décrivant l'événement tactile.
     * @return true si l'événement a été géré, false sinon.
     */
    @SuppressLint("ClickableViewAccessibility") // Ajout
    @Override
    public boolean onTouch(View v, MotionEvent event) {
        return gestureDetector.onTouchEvent(event);
    }

    /**
     * Classe interne qui étend GestureDetector.SimpleOnGestureListener pour gérer
     * spécifiquement les gestes de balayage (fling).
     */
    private final class GestureListener extends GestureDetector.SimpleOnGestureListener {

        private static final int SWIPE_THRESHOLD = 100;        // Distance minimale du swipe (en pixels).
        private static final int SWIPE_VELOCITY_THRESHOLD = 100;  // Vitesse minimale du swipe (en pixels par seconde).

        /**
         * Méthode appelée lorsqu'un appui initial sur l'écran est détecté.
         * On retourne toujours 'true' pour indiquer qu'on gère cet événement.
         *
         * @param e L'objet MotionEvent décrivant l'appui initial.
         * @return true, car on gère toujours l'événement 'onDown'.
         */
        @Override
        public boolean onDown(@NonNull MotionEvent e) { // Ajout @NonNull
            return true;
        }

        /**
         * Méthode appelée lorsqu'un geste de balayage (fling) est détecté.
         *
         * @param e1        L'objet MotionEvent du premier appui (début du swipe). Peut être null.
         * @param e2        L'objet MotionEvent de la fin du swipe.
         * @param velocityX La vitesse du swipe en pixels par seconde sur l'axe X.
         * @param velocityY La vitesse du swipe en pixels par seconde sur l'axe Y.
         * @return true si le geste de balayage a été géré, false sinon.
         */
        @Override
        public boolean onFling(MotionEvent e1, @NonNull MotionEvent e2, float velocityX, float velocityY) { // Ajout @NonNull
            // Vérification de nullité pour e1, nécessaire car onFling peut être appelé même si onDown retourne false ou si le geste est complexe.
            if (e1 == null) {
                return false;
            }

            boolean result = false;
            try {
                float diffY = e2.getY() - e1.getY();  // Différence de position sur l'axe Y.
                float diffX = e2.getX() - e1.getX();  // Différence de position sur l'axe X.

                // Détermine si le swipe est plutôt horizontal ou vertical.
                if (Math.abs(diffX) > Math.abs(diffY)) {
                    // Swipe horizontal.
                    if (Math.abs(diffX) > SWIPE_THRESHOLD && Math.abs(velocityX) > SWIPE_VELOCITY_THRESHOLD) {
                        if (diffX > 0) {
                            listener.onSwipeRight(); // Swipe vers la droite.
                        } else {
                            listener.onSwipeLeft();  // Swipe vers la gauche.
                        }
                        result = true;
                    }
                } else {
                    // Swipe vertical.
                    if (Math.abs(diffY) > SWIPE_THRESHOLD && Math.abs(velocityY) > SWIPE_VELOCITY_THRESHOLD) {
                        if (diffY > 0) {
                            listener.onSwipeBottom(); // Swipe vers le bas.
                        } else {
                            listener.onSwipeTop();    // Swipe vers le haut.
                        }
                        result = true;
                    }
                }
            } catch (Exception exception) {
                exception.fillInStackTrace(); // Gestion des erreurs (journalisation).
            }
            return result;
        }
    }
}